from pequod_agent.utilities.console import print_table

PREFIX = 'inspect_'


def get_funcs():
    funcs = {}
    for key, val in globals().items():
        if key.startswith(PREFIX):
            funcs[key[len(PREFIX):]] = val

    return funcs


def inspect(node_state, args):
    """
    Inspection helper: process args and print useful information such as running containers
    """
    funcs = get_funcs()

    cmd = ''.join(args[:1])

    if cmd:
        matching  = [f for f in funcs if f.startswith(cmd)]
        if len(matching) > 1:
            raise Exception('Ambiguous command "{}", multiple matches found: {}'.format(cmd, matching))
        elif len(matching) < 1:
            raise Exception('Command "{}" not found. Available commands are: {}'.format(cmd, sorted(funcs.keys())))
        command = funcs[matching[0]]
    else:
        command = inspect_instances

    command(node_state)


def inspect_help(node_state):
    """
    Show list of commands.
    :param node_state:
    :return:
    """
    COLS = 'command description'.split()
    rows = []
    for name, func in sorted(get_funcs().items()):
        rows.append({
            'command': name,
            'description': func.__doc__.split('.')[0].strip(),
        })
    print_table(COLS, rows)


def inspect_instances(node_state):
    """
    List all running application instances.
    """
    COLS = 'instance_name repository_name application_name application_version' \
           + 'zone_name pid ipv6 instance_started_time'.split()
    rows = []
    for instance in node_state.instances:
        rows.append({
            'instance_name': instance.name,
            'repository_name': instance.repository_name,
            'application_name': instance.application_name,
            'application_version': instance.application_version,
            'zone_name': instance.zone_name,
            'pid': instance.pid,
            'ipv6': instance.ip_address,
            'instance_started_time': instance.instance_started_time
        })
    print_table(COLS, rows)


def inspect_services(node_state):
    """
    List provided and required services of running instances.
    """
    COLS = 'instance_name repository_name application_name application_version' \
           + ' type service_name service_version local_hostname local_port'.split()
    rows = []
    for instance in node_state.instances:
        for service in instance.manifest.services:
            rows.append({
                'instance_name': instance.name,
                'repository_name': instance.repository_name,
                'application_name': instance.application_name,
                'application_version': instance.application_version,
                'type': service['type'],
                'service_name': service['service_name'],
                'service_version': service['service_version'],
                'local_hostname': service.get('local_hostname') or instance.ip_address,
                'local_port': service['local_port'],
            })
    print_table(COLS, rows)

