#!/usr/bin/env python3
from collections import namedtuple

HA_PROXY_RULE_TEMPLATE = """
frontend {name}:{version}
    bind {{proxy_ip}}:{local_port}
    default_backend {name}:{version}

backend {name}:{version}
    {backend}
    # we need to set the source IP to our container's IP for ip6tables
    source {{source_ip}}:0
"""


class Service:
    """
    Model service running on the cluster
    """
    def __init__(self, service_name, service_version, local_hostname: str, local_port, endpoints):
        self.name = service_name
        self.version = service_version
        self.local_ip = None
        self.local_hostname = local_hostname
        self.local_port = local_port
        self.endpoints = endpoints

    @property
    def haproxy_rule(self):
        backend = '\n'.join('    server {e.ip} {e}'.format(e=endpoint) for endpoint in self.endpoints).strip()
        service_rule = HA_PROXY_RULE_TEMPLATE.format(backend=backend, **vars(self))
        return service_rule


class ServiceEndpoint(namedtuple('ServiceEndpoint', 'ip port')):
    """
    Service endpoint extracted from the cloud state
    """
    def __str__(self):
        return '{ip}:{port}'.format(ip=self.ip, port=self.port)
